

#include <stdio.h>

/*  This code performs NITER iterations of an NxN life board.  */

#define N          64
#define NITER      50

int board [N] [N] ;

void dump() ;

void main(int argc, char **argv)
{
  int neighbours [N] [N] ;

  int x, y ;
  int iter ;

  /*  Initialise `board'.  */

  for(x = 0 ; x < N ; x++) {
    for(y = 0 ; y < N ; y++) {
      if(x == N / 2 || y == N / 2)
        board [x] [y] = 1 ;
      else
        board [x] [y] = 0 ;
    }
  }

  /*  Dump initial state of board.  */

  dump(0) ;

  /*  Main update loop.  */

  for(iter = 1 ; iter <= NITER ; iter++) {

    /*  Calculate a block of neighbour sums.  */

    for(x = 0 ; x < N ; x++) {
      int x_n = (x + N - 1) % N ;
      int x_p = (x + 1) % N ;

      for(y = 0 ; y < N ; y++) {
        int y_n = (y + N - 1) % N ;
        int y_p = (y + 1) % N ;

        neighbours [x] [y] =
            board [x_n] [y_n] + board [x_n] [y] + board [x_n] [y_p] +
            board [x]   [y_n]                   + board [x]   [y_p] +
            board [x_p] [y_n] + board [x_p] [y] + board [x_p] [y_p] ;
      }
    }
   
    /*  Update block of board values.  */

    for(x = 0 ; x < N ; x++) {
      for(y = 0 ; y < N ; y++) {
        int neighbour = neighbours [x] [y] ;

        if(neighbour < 2 || neighbour > 3)
          board [x] [y] = 0 ;
        if(neighbour == 3)
          board [x] [y] = 1 ;
      }
    }

    /*  Dump current state of board.  */

    dump(iter) ;
  }
}



void dump(int iter) {

  /*  Write current state of board to a `.pgm' file.  */

  char fname [256] ;
  FILE* f ;
  int x, y ;

  sprintf(fname, "life%2.2d.pgm", iter) ;

  f = fopen(fname, "w") ;

  fprintf(f, "P2\n%3d  %3d\n%3d\n", N, N, 1) ;

  for(x = 0 ; x < N ; x++) {
    for(y = 0 ; y < N ; y++)
      fprintf(f, "%d ", board [x] [y]) ;
    fprintf(f, "\n") ;
  }

  fclose(f) ;
}

